require 'roxie/version'
require 'roxie/extractor'
require 'yaml'

module Roxie

  # The root path for Roxie's source libraries
  ROOT = File.expand_path(File.dirname(__FILE__))

  # Languages that Roxie understands
  LANGUAGES = YAML.load_file(File.join(ROOT, 'roxie', 'languages.yml')).each do |name, lang|
    # Generate matchers for the language
    match = {}
    match[:start_single] = Regexp.new("^\s*#{Regexp.escape(lang['single'])} ") if lang['single']
    if lang['multi']
      match[:start_multi] = Regexp.new("^\s*#{Regexp.escape(lang['multi']['start'])}[ ]?") if lang['multi']['start']
      match[:middle_multi] = Regexp.new("^\s*#{Regexp.escape(lang['multi']['middle'])}[ ]?") if lang['multi']['middle']
      if lang['multi']['end']
        match[:end_multi] = Regexp.new("#{Regexp.escape(lang['multi']['end'])}")
        match[:end_multi_with] = Regexp.new("(?=#{Regexp.escape(lang['multi']['end'])}.*$)")
      end
    end

    lang[:matchers] ||= {}
    lang[:matchers].merge!(match)
  end.freeze

module_function

  # Extract comments from text
  def extract(text, options = {})
    Extractor.new(options).extract(text)
  end

end
