require 'roxie'
require 'thor'
require 'thor/group'

# Include the core CLI items
Dir[File.expand_path('../cli/*.rb', __FILE__)].each { |f| require f }

module Roxie
  module CLI

    class Base < Thor

      map ['-v', '--version'] => :version, 'd' => :doc

      class << self
        alias :old_register :register
        def register(klass, subcommand_name, usage, description, options={})
          # Keep track of 'subgroups' as well as subcommands
          self.subgroups << subcommand_name.to_s if klass <= Thor::Group
          old_register(klass, subcommand_name, usage, description, options)
        end

        def subgroups
          @subgroups ||= from_superclass(:subgroups, [])
        end

        alias :old_task_help :task_help
        def task_help(shell, command_name)
          meth = normalize_task_name(command_name)
          if subgroups.include?(meth)
            # Make sure to namespace all subcommands appropriately
            klass = Thor::Util.find_by_namespace(meth)
            klass.help(shell)
          else
            old_task_help(shell, command_name)
          end
        end
        alias :command_help :task_help
      end

      default_task :doc

      desc "version", "Prints Roxie's version information"
      def version
        say "Roxie #{Roxie::VERSION}"
        klass, task = Thor::Util.find_class_and_command_by_namespace("doc:doc")
      end

      def method_missing(meth, *args)
        # Assume the user typed in a file input for the
        # :doc command
        Doc.start(args.unshift(meth.to_s), :shell => self.shell)
      end

    end

    # Subcommands
    Base.register Doc, 'doc', "doc [INPUT]", "Generates documentation from input files"

  end
end
